@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile;
 use vars '$VERSION';
-$VERSION = '0.62';
+$VERSION = '0.63';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use XML::LibXML;
@@ -107,7 +107,7 @@ sub dataToXML($)
 
     if(my $known = $self->knownNamespace($thing))
     {   my $fn = $self->findSchemaFile($known)
-            or error __x"cannot find pre-installed name-space files named {path} for {name}"
+            or error __x"cannot find pre-installed name-space file named {path} for {name}"
                  , path => $known, name => $thing;
 
         return $self->_parseFile($fn);
