@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema::XmlWriter;
 use vars '$VERSION';
-$VERSION = '0.62';
+$VERSION = '0.63';
 
 use strict;
 use warnings;
@@ -13,7 +13,8 @@ no warnings 'once';
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use List::Util    qw/first/;
-use XML::Compile::Util qw/pack_type unpack_type odd_elements block_label/;
+use XML::Compile::Util qw/pack_type unpack_type odd_elements
+   block_label type_of_node/;
 
 
 # Each action implementation returns a code reference, which will be
@@ -345,6 +346,7 @@ sub complex_element
     my @elems = odd_elements @$elems;
     my @attrs = @$attrs;
     my @anya  = @$any_attr;
+    my $ignore_unused_tags = $args->{ignore_unused_tags};
 
     sub { my ($doc, $data) = @_;
           unless(UNIVERSAL::isa($data, 'HASH'))
@@ -368,7 +370,8 @@ sub complex_element
           if(my @not_used = sort keys %$copy)
           {   mistake __xn "tag `{tags}' not used at {path}"
                 , "unused tags {tags} at {path}"
-                , scalar @not_used, tags => \@not_used, path => $path;
+                , scalar @not_used, tags => \@not_used, path => $path
+                   unless $ignore_unused_tags;
           }
 
           my $node  = $doc->createElement($tag);
@@ -661,12 +664,11 @@ sub anyAttribute
               @$attrs or next;
 
               foreach my $node (@$attrs)
-              {   my $nodens = $node->namespaceURI;
-                  my $name   = $node->localName;
-                  next if $nodens eq $ns && $name eq $local;
+              {   my $nodetype = type_of_node $node;
+                  next if $nodetype eq $type;
 
                   error __x"provided 'anyAttribute' node has type {type}, but labeled with {other} at {path}"
-                     , type => packtype($nodens, $name), other => $type, path => $path
+                     , type => $nodetype, other => $type, path => $path
               }
 
               push @res, @$attrs;
@@ -705,11 +707,11 @@ sub anyElement
               {   my $nodens = $node->namespaceURI;
                   defined $nodens or next; # see README.todo work-around
 
-                  my $name   = $node->localName;
-                  next if $nodens eq $ns && $name eq $local;
+                  my $nodetype = type_of_node $node;
+                  next if $nodetype eq $type;
 
                   error __x"provided 'any' element node has type {type}, but labeled with {other} at {path}"
-                     , type => pack_type($nodens, $name), other => $type, path => $path
+                     , type => $nodetype, other => $type, path => $path
               }
 
               push @res, @$elems;
