@@ -7,14 +7,15 @@ use strict;
 
 package XML::Compile::Util;
 use vars '$VERSION';
-$VERSION = '0.62';
+$VERSION = '0.63';
 use base 'Exporter';
 
 my @constants  = qw/XMLNS SCHEMA1999 SCHEMA2000 SCHEMA2001 SCHEMA2001i/;
 our @EXPORT    = qw/pack_type unpack_type/;
 our @EXPORT_OK =
-   ( qw/pack_id unpack_id odd_elements block_label/
-   , @constants );
+  ( qw/pack_id unpack_id odd_elements block_label type_of_node/
+  , @constants
+  );
 our %EXPORT_TAGS = (constants => \@constants);
 
 use constant XMLNS       => 'http://www.w3.org/XML/1998/namespace';
@@ -57,4 +58,10 @@ sub block_label($$)
     $block_abbrev{$kind} . $label;
 }
 
+
+sub type_of_node($)
+{   my $node = shift or return ();
+    pack_type $node->namespaceURI, $node->localName;
+}
+
 1;
